%  CHANGE_CHROMA   Change the saturation of the image (CIELAB/CIELUV space).
%  
%  SYNOPSIS:
%   image_out = change_chroma(image_in, fraction, col)
% 
%  DEFAULTS:
%     frac : 1 (Nothing changes)
%     col  : 'LAB' 
% 
%  DESCRIPTION:
%     The chroma of the input image is scaled with a fraction. If this fraction
%     is larger than 1, the image gets more colorful. If the fraction is smaller 
%     than 1, the image gets desaturated. This manipulation can be done in 
%     CIELAB, CIELUV  and RGB space.
%     Note that with a fraction larger than 1 the number of out-of-gamut pixels
%     increases.
% 
%  EXAMPLE:
%     a = change_chroma( readcolorim, 2)

% (C) Copyright 1999-2002               Pattern Recognition Group
%     All rights reserved               Faculty of Applied Physics
%                                       Delft University of Technology
%                                       Lorentzweg 1
%                                       2628 CJ Delft
%                                       The Netherlands
%
% Judith Dijk, June 2002.

function out = change_chroma( in, frac, col)

if nargin < 1, error('Need an input argument.'); end
if nargin < 2, frac = 1; end % Nothing changes
if nargin < 3, col = 'L*a*b*'; end;

col_in = colorspace( in);
col = color_alias( col);
if( ~any(strcmp(col,known_colorspaces)))
   col = 'L*a*b*';
end
if( ~iscolor(in)) error('The input image should be a color image.'); end;  

if strcmp(col, 'RGB')
   rgb_in = colorspace(in, 'RGB');
   Y = colorspace( rgb_in, 'grey')./255;
   out = frac * rgb_in + (1-frac) * 255 * Y;
   out = joinchannels(col, out(1), out(2), out(3));
   for i=1:3
      out(i).color = rgb_in(1).color;
   end
else   
   ori_col = colorspace(in);
   lab_in = colorspace(in, col); % lab_in could also be in luv space....
   
   out = lab_in;
   out(2) = out(2).*frac;
   out(3) = out(3).*frac;
   
   out = joinchannels(col, out(1), out(2), out(3));
   for i=1:3
      out(i).color = lab_in(1).color;
   end
end   
out = colorspace(out, col_in);
   
function col = color_alias(col)
switch lower(col)
   case {'lab','cielab','l*a*b*'}
      col = 'L*a*b*';
   case {'luv','cieluv','l*u*v*'}
      col = 'L*u*v*';
   case {'rgb','RGB'}
      col = 'RGB';
   otherwise
      col = col;
end
function list = known_colorspaces
list = {'L*a*b*', 'L*u*v*', 'RGB'}; % 'Yxy'};



		
