%  CHANGE_GAMMA  Change the gamma of the input image.
%
%  SYNOPSIS:
%     image_out = change_gamma(image_in, gammaval, functionstyle, center_point,
%                 colorspace)
%
%  DEFAULTS:
%     gammaval       : 1  (nothing happens)
%     functionstyle  : 1
%     center_point   : (max+min)/2
%     colorspace     : the colorspace of the input image. If this is not a
%                      correct colorspace, the colorspace is 'LAB'
%
%  DESCRIPTION:
%     In this function the input image is changed with a gamma manipulation.
%
%     There are three different functionstyles:
%     1.  out = min + (max-min)*((in-min)/(max-min))^gammaval
%     2.  center = (max+min)/2
%            out = min + (center-min) * ((in-min)./(center-min))^gammaval
%                 for in < center
%            out = max + (center-max) * ((in-max)./(center-max))^gammaval
%                 for in >= center
%     3.  the same as 2, but now with a given center_point. This center_point
%           may differ for the different pixel values.
%
%     There are 5 different colorspaces in which this function can be performed:
%     1.  CIELAB: gamma manipulation of the Lightness L (a and b constant)
%     2.  CIELUV: gamma manipulation of the Lightness L (u and v constant)
%     3.  XYZ/Yxy: gamma manipulation of the luminance Y (x and y constant)
%     4.  RGB: gamma manipulation of R, G and B. Different gamma values can be
%           given for the different planes.
%     5.  GREY: gamma manipulation of the grey value image
%
%     Note that the center_point definition only makes sense for functionstyle 3
%     Otherwise it is ignored.
%%
%  EXAMPLE: a = change_gamma( readcolorim, 2)
%           a = change_gamma( readcolorim, 2, 1, [], 'RGB')
%
%

% (C) Copyright 1999-2002               Pattern Recognition Group
%     All rights reserved               Faculty of Applied Physics
%                                       Delft University of Technology
%                                       Lorentzweg 1
%                                       2628 CJ Delft
%                                       The Netherlands
%
% Judith Dijk, June 2002.


function [out] = change_gamma( in, gamma, functionstyle, meanim, col)

if nargin < 1, error('Need an input argument.'); end
if nargin < 2, gamma = 1; end % er veranderd niets
if nargin < 3, functionstyle = 1; end % fprintf(1, 'meanim = %d, functionstyle = %d \n', meanim, functionstyle);
if (nargin < 4 |isempty(meanim)) & functionstyle == 3, functionstyle = 2; end %
if( nargin < 4), meanim = []; end
if nargin < 5, col = colorspace(in); end

col = color_alias( col);
if( ~any(strcmp(col,known_colorspaces)))
   col = 'L*a*b*';
end
if ~strcmp(col, 'RGB')
   if prod(size(gamma)) > 1 error('gamma can only be one value for this colorspace'); end;
else
   if prod(size(gamma)) ~= 3
      error('gamma should have 1 or 3 values.');
   end
end

if( ~iscolor(in))
   out = change_gamma_grey(in, gamma, functionstyle, meanim);
elseif( strcmp(col,'RGB'))
   out = change_gamma_rgb(in, gamma, functionstyle, meanim);
else
   out = change_gamma_L(in, gamma, functionstyle, meanim, col);
end


function list = known_colorspaces
list = {'L*a*b*','L*u*v*','RGB', 'Yxy'};

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [out] = change_gamma_grey( in, gamma, functionstyle, meanim)

if (nargin < 4 | isempty(meanim)) & functionstyle == 3, functionstyle = 2; end %


fprintf(1, 'The conversion colorspace = grey, because you gave a grey valued input image\n');

image_L = in;

switch ( functionstyle )
   case 1
      maxim = max(image_L);
      minim = min(image_L);
      image_L = minim + (maxim-minim).*((image_L-minim)./(maxim-minim)).^gamma;
   case 2
      maxim = max(max( image_L));
      minim = min(min( image_L));
      meanim = (maxim-minim)./2+minim;

      image_L_out1 = minim + (meanim-minim).*((image_L-minim)./(meanim-minim)).^gamma;
      image_L_out2 = maxim + (meanim-maxim).*((image_L-maxim)./(meanim-maxim)).^gamma;
      image_L = (image_L_out1).*(image_L < meanim) + (image_L_out2).*(image_L >= meanim);
   case 3
      maxim = max(max( image_L));
      minim = min(min( image_L));

      image_L_out1 = minim + (meanim-minim).*((image_L-minim)./(meanim-minim)).^gamma;
      image_L_out2 = maxim + (meanim-maxim).*((image_L-maxim)./(meanim-maxim)).^gamma;
      image_L = (image_L_out1).*(image_L < meanim) + (image_L_out2).*(image_L >= meanim);
end

out = image_L;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [out] = change_gamma_rgb( in, gamma, functionstyle, meanim)

if (nargin < 4 | isempty(meanim)) & functionstyle == 3, functionstyle = 2; end %

if (prod(size(gamma)) == 1)
   gammaR = gamma; gammaG = gamma; gammaB = gamma;
else
   gammaR = gamma(1); gammaG = gamma(2); gammaB = gamma(3);
end
% the check for one or three values is already performed in change_gamma
col = 'RGB';
fprintf(1, 'The conversion colorspace = %s\n', col);
ori_col = colorspace(in);
rgb_in = colorspace(in, col);

%image_R = rgb_in(1);
%image_G = rgb_in(2);
%image_B = rgb_in(3);
%image_R(image_R < 0) = 0;
%image_G(image_G < 0) = 0;
%image_B(image_B < 0) = 0;
rgb_in(1).data(rgb_in(1).data<0) = 0;
rgb_in(2).data(rgb_in(2).data<0) = 0;
rgb_in(3).data(rgb_in(3).data<0) = 0;
out = rgb_in;

switch ( functionstyle )
   case 1
      out(1) = (rgb_in(1)./255).^gammaR.*255;
      out(2) = (rgb_in(2)./255).^gammaG.*255;
      out(3) = (rgb_in(3)./255).^gammaB.*255;
   case 2
      maxim = 255;
      minim = 0;
      meanim = 127.5;

      image_R_out1 = minim + (meanim-minim).*((rgb_in(1)-minim)./(meanim-minim)).^gammaR;
      image_R_out2 = maxim + (meanim-maxim).*((rgb_in(1)-maxim)./(meanim-maxim)).^gammaR;
      out(1) = (image_R_out1).*(rgb_in(1) < meanim) + (image_R_out2).*(rgb_in(1) >= meanim);

      image_G_out1 = minim + (meanim-minim).*((rgb_in(2)-minim)./(meanim-minim)).^gammaG;
      image_G_out2 = maxim + (meanim-maxim).*((rgb_in(2)-maxim)./(meanim-maxim)).^gammaG;
      out(2) = (image_G_out1).*(rgb_in(2) < meanim) + (image_G_out2).*(rgb_in(2) >= meanim);

      image_B_out1 = minim + (meanim-minim).*((rgb_in(3)-minim)./(meanim-minim)).^gammaB;
      image_B_out2 = maxim + (meanim-maxim).*((rgb_in(3)-maxim)./(meanim-maxim)).^gammaB;
      out(3) = (image_B_out1).*(rgb_in(3) < meanim) + (image_B_out2).*(rgb_in(3) >= meanim);
   case 3
      maxim = 255;
      minim = 0;

      image_R_out1 = minim + (meanim-minim).*((rgb_in(1)-minim)./(meanim-minim)).^gammaR;
      image_R_out2 = maxim + (meanim-maxim).*((rgb_in(1)-maxim)./(meanim-maxim)).^gammaR;
      out(1) = (image_R_out1).*(rgb_in(1) < meanim) + (image_R_out2).*(rgb_in(1) >= meanim);

      image_G_out1 = minim + (meanim-minim).*((rgb_in(2)-minim)./(meanim-minim)).^gammaG;
      image_G_out2 = maxim + (meanim-maxim).*((rgb_in(2)-maxim)./(meanim-maxim)).^gammaG;
      out(2) = (image_G_out1).*(rgb_in(2) < meanim) + (image_G_out2).*(rgb_in(2) >= meanim);

      image_B_out1 = minim + (meanim-minim).*((rgb_in(3)-minim)./(meanim-minim)).^gammaB;
      image_B_out2 = maxim + (meanim-maxim).*((rgb_in(3)-maxim)./(meanim-maxim)).^gammaB;
      out(3) = (image_B_out1).*(rgb_in(3) < meanim) + (image_B_out2).*(rgb_in(3) >= meanim);
end

for i=1:3
   out(i).color = rgb_in(1).color;
end
out = colorspace(out,ori_col);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [out] = change_gamma_L( in, gamma, functionstyle, meanim, col)

if (nargin < 4 | isempty(meanim)) & functionstyle == 3, functionstyle = 2; end %

fprintf(1, 'The conversion colorspace = %s\n', col);
ori_col = colorspace(in);
lab_luv = colorspace(in, col);

image_L = lab_luv(1);

switch ( functionstyle )
   case 1
      maxim = max(image_L);
      minim = min(image_L);
      image_L = minim + (maxim-minim).*((image_L-minim)./(maxim-minim)).^gamma;
   case 2
      maxim = max(max( image_L));
      minim = min(min( image_L));
      meanim = (maxim-minim)./2+minim;

      x = (image_L-minim)./(meanim-minim);
      x = (x>0)*x + (x<=0)*0;
      image_L_out1 = minim + (meanim-minim).*x.^gamma;
      x = (image_L-maxim)./(meanim-maxim);
      x = (x>0)*x + (x<=0)*0;
      image_L_out2 = maxim + (meanim-maxim).*x.^gamma;
      image_L = (image_L_out1).*(image_L < meanim) + (image_L_out2).*(image_L >= meanim);
   case 3
      maxim = max(max( image_L));
      minim = min(min( image_L));
      fprintf(1, 'maxim = %f, minim = %f, meanim = %f\n', maxim, minim, meanim);
      if( meanim < minim )
         meanim = minim;
         fprintf(1, 'The center point is chosen badly.\n');
         fprintf(1, 'The maximum = %f, the minimum = %f.\n', maxim, minim);
         fprintf(1, 'This is a gamma change of functionstyle 1 with gamma = %f\n', 1./gamma);
      end
      if( meanim > maxim )
         meanim = maxim;
         fprintf(1, 'The center point is chosen badly.\n');
         fprintf(1, 'The maximum = %f, the minimum = %f.\n', maxim, minim);
         fprintf(1, 'This is a gamma change of functionstyle 1 with gamma = %f\n', gamma);
      end

      if( meanim ~= minim )
         x = (image_L-minim)./(meanim-minim);
         x = (x>0)*x + (x<=0)*0;
         image_L_out1 = minim + (meanim-minim).*x.^gamma;
      else
         image_L_out1 = image_L.*0;
      end
      if( meanim ~= maxim )
         x = (image_L-maxim)./(meanim-maxim);
         x = (x>0)*x + (x<=0)*0;
         image_L_out2 = maxim + (meanim-maxim).*x.^gamma;
      else
         image_L_out2 = image_L.*0;
      end
      image_L = (image_L_out1).*(image_L < meanim) + (image_L_out2).*(image_L >= meanim);
end

out = dip_image('array',[3,1]);
out(1) = image_L(1);
out(2) = lab_luv(2);
out(3) = lab_luv(3);
for i=1:3
   out(i).color = lab_luv(1).color;
end

out = colorspace(out, ori_col );


function col = color_alias(col)
switch lower(col)
   case {'lab','cielab','l*a*b*'}
      col = 'L*a*b*';
   case {'luv','cieluv','l*u*v*'}
      col = 'L*u*v*';
   case 'rgb'
      col = 'RGB';
   case 'xyz'
      col = 'Yxy';
   case 'yxy'
      col = 'Yxy';
   otherwise
      col = col;
end
