%  GAMUT_MAPPING  Map the pixels of the input image on the gamut of the output
%     device.
%
%  SYNOPSIS:
%     mage_out = gamut_mapping(image_in, clipping_style, gamut_data, param)
% 
%  DEFAULTS:
%     clipping_style : 'ort_clip'
%     gamut_data     : standard RGB gamut (D65, 2 degrees observer)
%     param:         : chord_point: [0 50] ([Chroma Lightness])
%                      scaling factor: 1
%                      percentage: 0
% 
%  DESCRIPTION:
%     The goal of this function is to reduce the out-of-gamut pixels of an
%     image. This can be achieved in two different ways: by clipping all
%     out-of-gamut pixels on the boundary of the gamut, or by compression of the
%     colors of all pixel points. The advantage of clipping over compression is
%     that only pixels that need to be changed are changed, so that the chroma
%     and lightness of the image are maximal kept. The advantage of compression
%     over clipping is that color differences in out-of-gamut colors can still
%     be seen. It depends on the image and the number of out-of-gamut colors
%     which mapping style gives the best results.
%
%     Currently 6 types of gamut mapping are implemented. For all
%     types the hue h remains constant.
%     1.  CHORD CLIPPING: clipping over a constant line towards the point
%           chord point. The chord point can be defined using param(1) and
%           param(2).
%     2.  ORTHOGONAL CLIPPING: clipping to the closest boundary point (in 
%           (chroma/lightness space)  
%     3.  CHORD COMPRESSION: compression of lightness and chroma towards the
%           chord point. The compression factor is determined by the
%           maximum scaling that is needed to get rid of all out-of-gamut
%           points. This maximum can be scaled by the user with param(3).
%           param(1) and param(2) are used for the chroma and lightness of the
%           chord point, respectively
%     4.  CHROMA COMPRESSION: compression of the chroma while the lightness
%           remains constant. The compression factor is determined by the
%           maximum scaling that is needed to get rid of all out-of-gamut
%           points. This maximum can be scaled by the user with param(1).
%     5.  SIMPLE CHORD COMPRESSION: compression of the lightness and the chroma
%           towards the chord point. The scaling factor is given by the user.
%     6.  SIMPLE CHROMA COMPRESSION: compression of the lightness. The compression
%           factor is directly given by the user. 
%
%
%  EXAMPLE : a = gamut_mapping( change_chroma( readcolorim, 2))

% (C) Copyright 1999-2002               Pattern Recognition Group
%     All rights reserved               Faculty of Applied Physics
%                                       Delft University of Technology
%                                       Lorentzweg 1
%                                       2628 CJ Delft
%                                       The Netherlands
%
% Judith Dijk, June 2002.

function out = gamut_mapping(in, clipping_function, gamut_bound, param)

if nargin < 1, error('Need an input argument.'); end

if( nargin < 2 | isempty(clipping_function) )
   clipping_function = 'ort_clip';
end
clipping_function = clipping_alias( clipping_function );
if( ~any(strcmp(clipping_function, known_clipping_functions)))
   fprintf(1, 'this clipping function is not known\n');
   fprintf(1, 'the used clipping function is ort_clip\n');
   clipping_function = 'ort_clip';
end

if( nargin < 3)
   gamut_bound = make_gamut;
end

ori_col = colorspace(in);
ori_xyz = get_xyz(in);
in = colorspace(in, 'lab');

if( strcmp(clipping_function, 'chroma_map_simple') | ...
      strcmp(clipping_function, 'chord_map_simple'))
   if( strcmp(clipping_function, 'chroma_map_simple'))
      if nargin < 4
         scal = 1;
      else
         scal = param(1);
      end
      out = in;
      out(2) = in(2).*frac;
      out(3) = in(3).*frac;     
   else
      if nargin < 4
         scal = 1;
         chord = [0 50];
      else
         chord = param(1:2);
         scal = param(3);
         percen = param(4);
      end
      L = in(1);
      C = sqrt(in(2).^2 + in(3).^2);
      h = atan2(in(3), in(2));
      C_new = (C - chord(1)).*scal + chord(1);
      L_new = (L - chord(2)).*scal + chord(2);
      
      out = in;
      out(1) = L_new;
      out(2) = C_new .* cos(h);
      out(3) = C_new .* sin(h);          
   end
else
   % preparation for the actual gamut mapping procedure: the input image is 
   % reshaped in a 1-D image, in which the columns give the Lab of the points. 
 
   input_image = double(in(1));
   input_image(:, :, 2) = double(in(2));
   input_image(:, :, 3) = double(in(3));
 
   s = size( input_image);
   input_image = reshape( input_image, s(1).*s(2), 3);
 
   % find the correct mapping function. for explanation of the functions, see
   % above or the corresponding c-file.  
   switch ( clipping_function )
      case 'chord_clip'
         if nargin < 4
            chord = [0 50];
         elseif( size(param, 1).*size(param, 2) ~= 2)
            chord = [0 50];
         else
            chord = param(1);
         end      
         output_image = gamut_mapping_chord(input_image', gamut_bound', ....
            chord(1), chord(2));
      case 'ort_clip'
         output_image = gamut_mapping_ort(input_image', gamut_bound');
      case 'chroma_map'
         if( nargin < 4)
            scal = 1;
            percen = 0.050;
         else
            scal = param(1);
            percen = param(2);
         end
         minval_in = min(in(1));
         maxval_in = max(in(1));
		   input_image = scale_lightness(input_image, minval_in, maxval_in, gamut_bound);
         output_image = gamut_compression_chord(input_image', gamut_bound', ...
            0, 0, 0, scal, percen);
	   case 'chord_map'
         if nargin < 4
            chord = [0 50];
            scal = 1;
            percen = 0.050;
         elseif( size(param, 1).*size(param, 2) ~= 4)
            chord = [0 50];
            scal = 1;
            percen = 0.050;
         else
            chord = param(1:2);
            scal = param(3);
            percen = param(4);
         end         
         fprintf(1, 'chord compression\n');
		   output_image = gamut_compression_chord(input_image', gamut_bound', ...
		   	chord(1), chord(2), 1, scal, percen);
      otherwise
      error('this clipping style is not known');
   end   
   
   % restore the correct dimensions of the image.
   out = output_image;
   out = reshape(out, 3, s(1).*s(2));
   out = reshape(out', s(1), s(2), 3);
   out1 = dip_image( squeeze(out(:, :, 1)));
   out2 = dip_image( squeeze(out(:, :, 2)));
   out3 = dip_image( squeeze(out(:, :, 3)));

   output_image = joinchannels( 'lab', out1, out2, out3);
end

% restore the image information
for i=1:3
   output_image(i).color = in(1).color;
end
out = colorspace(output_image, ori_col );


function list = known_clipping_functions % only clipping is relevant here
list = {'ort_clip','chord_clip', 'chroma_map', 'chord_map', ...
   'chroma_map_simple', 'chord_map_simple'};


function clipping = clipping_alias(clipping)
switch lower(clipping)
   case {'ort', 'ortogonal'}
      clipping = 'ort_clip';
   case {'chord', '2point'}
      clipping = 'chord_clip';
	case {'chord_mapping', 'chord_map', 'mapping'}
		clipping = 'chord_map';
	case {'chroma_mapping', 'chroma_map'}
		clipping = 'chroma_map';
    otherwise
      clipping = clipping;
end

%% used in chroma mapping
function out = scale_lightness(in, minval_in, maxval_in, gamut_bound)

if( nargin < 4)
   minval = 0;
   maxval = 100;
else
   lightness_vals = [gamut_bound(:, 1); gamut_bound(:, 4); gamut_bound(:, 7)];
   maxval = max(lightness_vals);
   labvals = [gamut_bound(:, 1:3); gamut_bound(:, 4:6); gamut_bound(:, 7:9)];
   io = find( labvals(:, 1) < 50 & abs(labvals(:, 2)) < 1 & ...
       abs( labvals(:, 3))< 1);
   minval = max( labvals(io, 1));
end

% minval_in = min(in(:, 1))
% maxval_in = max(in(:, 1));

minval_in = 0;
maxval_in = 99.5433;
minval = 32.3831;
maxval = 100;

out = in;
% fprintf(1, '%f %f %f %f\n', minval_in, minval, maxval_in, maxval);
if( minval_in < minval | maxval_in > maxval)
   if( maxval_in ~= minval_in)
      out(:, 1)= (in(:, 1)-minval_in)./(maxval_in-minval_in).*(maxval-minval) + minval;
   else
      if( maxval_in < minval)
         out(:, 1) = minval;
      elseif( maxval_in > maxval)
         out(:, 1) = maxval;
      else
         out(:, 1) = in(:, 1);
      end
   end
end
