%  GAMUT_STRETCH transforming the gamut from its original shape to ideal lines
%
%  SYNOPSIS:
%     [image_out gamut_lines gamut_data] = gamut_stretch(image_in, ...
%         device_values, reference_values)
% 
%  DEFAULTS:
%     device_values, reference_values : from standard RGB cube.
%
%  DESCRIPTION:
%     In this function the gamut is stretched in the LC plane.
%     For each pixel point the plane with a constant hue through the gamut is   
%     determined. In these plane all points are manipulated in such a way, that
%     the original shape is transformed into an ideal triangle. This triangle is
%     formed by the lines through the minimum luminance, the maximum luminance 
%     and the maximum chroma (Cusp point) All points keep their original 
%     chroma, and are scaled in the luminance direction such that the line 
%     through the cusp point remains at its position. All other luminances are 
%     scaled linearly between the gamut boundary and the line through the cusp 
%     point. 
%
%  SEE ALSO: gamut_destretch, gamut_sharpening, gamut_gamma
%   

% (C) Copyright 1999-2002               Pattern Recognition Group
%     All rights reserved               Faculty of Applied Physics
%                                       Delft University of Technology
%                                       Lorentzweg 1
%                                       2628 CJ Delft
%                                       The Netherlands
%
% Judith Dijk, 5 October 2001, adjusted 3 June 2002.

function [out, gamut_data_lines, gamut_data] = gamut_stretch(in, device_vals, ref_vals)

if( nargin < 2)
   gamut_data = make_gamut;
else
   gamut_data = make_gamut(device_vals, ref_vals);
end

if ~iscolor(in), error('Input image should be a color image'); end

ori_col = colorspace(in);
% if ~isfield( in(1).color, 'xyz')
%    in = set_xyz(in); 
% end
in = colorspace(in, 'lab');

L_im = double(in(1));
a_im = double(in(2));
b_im = double(in(3)); 

% This is the c procedure of interest. All triangles are checked to see if they
% cross the hue plane, if so, it is checked whether they are on the correct
% line.

[L_out, cuspL, cuspC, max_offset, min_offset] =  ...
   gamut_stretch_forward( L_im, a_im, b_im, gamut_data');

gamut_data_lines(1) = dip_image(cuspL);
gamut_data_lines(2) = dip_image(cuspC);
gamut_data_lines(3) = dip_image(max_offset);
gamut_data_lines(4) = dip_image(min_offset);

out = in;
out = joinchannels( 'lab', L_out, a_im, b_im);
for i=1:3
   out(i).color = in(1).color;
   out(i).color.space = 'L*a*b*';
end
out = colorspace(out, ori_col);
