%  PRINT_CMY2XYZ   cmy to xyz transformation for a calibrated printer
% 
%  SYNOPSIS:
%     image_out = print_cmy2xyz(image_in, cmy_ref, xyz_ref)
% 
%  DEFAULTS:
% 
%  DESCRIPTION:
%  This function is used to transform CMY values to XYZ values using a
%  (measured?) interpolation table. The values in cmy_ref should be equally
%  spaced over the CMY colorspace.
% 
%  EXAMPLE:
%
   
% (C) Copyright 1999-2002               Pattern Recognition Group
%     All rights reserved               Faculty of Applied Physics
%                                       Delft University of Technology
%                                       Lorentzweg 1
%                                       2628 CJ Delft
%                                       The Netherlands
%
% Judith Dijk, July 2002.

function out = print_cmy2xyz( in, cmy_ref, xyz_ref)

if nargin < 1, error('Need an input image'); end
if ~iscolor(in), error('Need a color input image'); end
if ~strcmp( colorspace( in), 'CMY') & ~strcmp( colorspace( in, 'CMYK')), 
   error('The input image should be in the CMY(K) colorspace'); end
if ~strcmp( colorspace( cmy_ref), 'CMY'), 
   error('The CMY reference values should be in the CMY colorspace'); end
if ~strcmp( colorspace( xyz_ref), 'XYZ'), 
   error('The XYZ reference values should be in the XYZ colorspace'); end
if nargin < 3
   error('Not enough input arguments'); end

in = colorspace(in, 'CMY');

num_points = size( xyz_ref(1), 1).*size(xyz_ref(1), 2);
num_points_cmy = size( cmy_ref(1), 1).*size(cmy_ref(1), 2);
if( num_points ~= num_points_cmy)
   error('The two reference images should have equal dimensions');
end

num_points = num_points.^(1./3);
if( abs(num_points - round(num_points)) > 0.0001)
   error( 'the reference image does not have correct dimensions');
end

in1 = double(in(1));
in2 = double(in(2));
in3 = double(in(3));

xyz_double1 = double( xyz_ref(1));
xyz_double2 = double( xyz_ref(2));
xyz_double3 = double( xyz_ref(3));
cmy_double1 = double( cmy_ref(1));
cmy_double2 = double( cmy_ref(2));
cmy_double3 = double( cmy_ref(3));

matrix_in = [cmy_double1(:), cmy_double2(:), cmy_double3(:), ... 
   xyz_double1(:), xyz_double2(:), xyz_double3(:)];

[out1, out2, out3] = lin_interpol_3( in1, in2, in3, matrix_in);

out = joinchannels( 'XYZ',  dip_image(out1), dip_image(out2), dip_image(out3));
for i=1:3
   out(i).color = in(1).color;
   out(i).color.space = 'XYZ';
end
