% RGB_TO_BORDER:  determines points on the gamut boundary in a certain direction
%
% SYNOPSIS
%   [image_up, image_down] = rgb_to_border(image_in, direction)
%
% DEFAULTS:
%   direction = 1
%
% DESCRIPTION: 
%   For each pixel in the image the color on the upper gamut boundary (image_up)
%   and the color on the lower gamut boundary (image_down) are determined. 
%   There are two directions implemented:
%   1:   The direction of constant hue and chroma, so the only difference 
%        between image_in, image_up and image_down is their luminance value.
%        Note that in RGB representation, in each pixel of image_up, R, G, or B 
%        is 255, whereas in image_down, one of the color values is 0.
%   2:   The direction of constant chroma. image_down is always (0, 0, 0)
%
% EXAMPLE 
%   [a, b] = rgb_to_border( readcolorim )

% (C) Copyright 1999-2002               Pattern Recognition Group
%     All rights reserved               Faculty of Applied Physics
%                                       Delft University of Technology
%                                       Lorentzweg 1
%                                       2628 CJ Delft
%                                       The Netherlands
%
% Piet W. Verbeek
% implemented in dipimage: Judith Dijk, July 2002.

function [ up, down] = rgb_to_border( in, direction)

if nargin < 1, error('Need an input argument.'); end
if ~iscolor(in), error('the input image should be a colorimage'); end 
if nargin < 2, direction = 1; end

ori_col = colorspace(in);
in = colorspace(in, 'RGB');

if direction == 1
   Largest=max(in(1),max(in(2),in(3)));
   Smallest=min(in(1),min(in(2),in(3)));
   up=in; % declaration
   up(1)=in(1)+255-Largest;
   up(2)=in(2)+255-Largest;
   up(3)=in(3)+255-Largest;
   up = joinchannels( 'RGB', up(1), up(2), up(3));
% up: same Hue and Chroma, but Value=255 
   down=in; % declaration
   down(1)=in(1)-Smallest;
   down(2)=in(2)-Smallest;
   down(3)=in(3)-Smallest;
   down = joinchannels('RGB', down(1), down(2), down(3));
% down: same Hue and Chroma, but Smallest=0 
else
   Value = max(max(in(1),in(2)),in(3));
   Value = Value.*(Value>0.001) + (Value <= 0.001);
   up = in; % declaration
   up(1) = 255*in(1)/Value;
   up(2) = 255*in(2)/Value;
   up(3) = 255*in(3)/Value;
   up(1) = out(1).*(Value>=0.001);
   up(2) = out(2).*(Value>=0.001);
   up(3) = out(3).*(Value>=0.001);
   down = up*0;
end

up = joinchannels( 'RGB', up(1), up(2), up(3));
for i=1:3
   up(i).color = in(i).color;
   up(i).color.space = 'RGB';
end
up = colorspace(up, ori_col );

down = joinchannels( 'RGB', down(1), down(2), down(3));
for i=1:3
   down(i).color = in(i).color;
   down(i).color.space = 'RGB';
end
down = colorspace(down, ori_col );

