%  SCAN_XYZ2RGB   xyz to rgb transformation for a calibrated scanner
% 
%  SYNOPSIS:
%     image_out = scan_xyz2rgb(image_in, rgb_xyz)
% 
%  DEFAULTS:
%     rgb_xyz : [3.50969    -1.74031  -0.545358
%					  -1.06828     1.97725   0.0345393
%					   0.0552852  -0.196645  1.05062]^(-1);
% 
%  DESCRIPTION:
%     This function is used to transform RGB values to XYZ values using a 
%     calibrated scanner model. This model is a first, second or third order 
%     polynomial. The transformation matrix can be made with
%     scanner_calibration.
% 
%  EXAMPLE:
%
   
% (C) Copyright 1999-2002               Pattern Recognition Group
%     All rights reserved               Faculty of Applied Physics
%                                       Delft University of Technology
%                                       Lorentzweg 1
%                                       2628 CJ Delft
%                                       The Netherlands
%
% Judith Dijk, July 2002.

function out = scan_rgb2xyz( in, rgb2xyz_mat)

if nargin < 1, error('Need an input image'); end
if ~iscolor(in), error('Need a color input image'); end
if ~strcmp( colorspace(in), 'RGB'), 
   error('The input image should be in the RGB colorspace'); end

if( nargin < 2)
	rgb2xyz_mat = [3.50969  -1.74031  -0.545358
						-1.06828  1.97725  0.0345393
						0.0552852  -0.196645  1.05062]^(-1);
end

out = dip_image('array',[3,1]);
out(:) = in; % to make sure that the array is a column vector.

out(1) = out(1)./255;
out(2) = out(2)./255;
out(3) = out(3)./255;

if( size(rgb2xyz_mat, 2) == 3)
   out = rgb2xyz_mat * out;
elseif( size(rgb2xyz_mat, 2) == 9)
   R = out(1);
   G = out(2);
   B = out(3);
   x = rgb2xyz_mat(1, :);
   out(1) = x(1).*R + x(2).*G + x(3).*B  + ...
       x(4).*R.^2 + x(5).*G.^2 + x(6).*B.^2 + ...
       x(7).*R.*G + x(8).*R.*B + x(9).*G.*B;
   x = rgb2xyz_mat(2, :);
   out(2) = x(1).*R + x(2).*G + x(3).*B  + ...
       x(4).*R.^2 + x(5).*G.^2 + x(6).*B.^2 + ...
       x(7).*R.*G + x(8).*R.*B + x(9).*G.*B;
   x = rgb2xyz_mat(3, :);
   out(3) = x(1).*R + x(2).*G + x(3).*B  + ...
       x(4).*R.^2 + x(5).*G.^2 + x(6).*B.^2 + ...
       x(7).*R.*G + x(8).*R.*B + x(9).*G.*B;
elseif( size(rgb2xyz_mat, 2) == 19)
   R = out(1);
   G = out(2);
   B = out(3);
   x = rgb2xyz_mat(1, :);
   out(1) = x(1).*R + x(2).*G + x(3).*B  + ...
      x(4).*R.^2 + x(5).*G.^2 + x(6).*B.^2 + ...
      x(7).*R.*G + x(8).*R.*B + x(9).*G.*B + ...
      x(10).*R.^3 + x(11).*G.^3 + x(12).*B.^3 + ...
      x(13).*R.^2.*G + x(14).*R.^2.*B + x(15).*G.^2.*R +...
      x(16).*G.^2.*B + x(17).*B.^2.*R + x(18).*B.^2.*G + ...
      x(19).*R.*G.*B;
   x = rgb2xyz_mat(2, :);
   out(2) = x(1).*R + x(2).*G + x(3).*B  + ...
      x(4).*R.^2 + x(5).*G.^2 + x(6).*B.^2 + ...
      x(7).*R.*G + x(8).*R.*B + x(9).*G.*B + ...
      x(10).*R.^3 + x(11).*G.^3 + x(12).*B.^3 + ...
      x(13).*R.^2.*G + x(14).*R.^2.*B + x(15).*G.^2.*R +...
      x(16).*G.^2.*B + x(17).*B.^2.*R + x(18).*B.^2.*G + ...
      x(19).*R.*G.*B;
   x = rgb2xyz_mat(3, :);
   out(3) = x(1).*R + x(2).*G + x(3).*B  + ...
      x(4).*R.^2 + x(5).*G.^2 + x(6).*B.^2 + ...
      x(7).*R.*G + x(8).*R.*B + x(9).*G.*B + ...
      x(10).*R.^3 + x(11).*G.^3 + x(12).*B.^3 + ...
      x(13).*R.^2.*G + x(14).*R.^2.*B + x(15).*G.^2.*R +...
      x(16).*G.^2.*B + x(17).*B.^2.*R + x(18).*B.^2.*G + ...
      x(19).*R.*G.*B;
else
   error('the transformation matrix does not have the right dimensions');
end
out = joinchannels( 'XYZ', out(1), out(2), out(3));
for i=1:3
   out(i).color = in(1).color;
   out(i).color.space = 'XYZ';
end
