%DIPMENUS   Not a user function
%   DIPMENUS is used by DIPIMAGE to determine the location of the toolbox
%   functions in the menu system.
%
%   To give your own functions a place in the menus system, create a
%   function called LOCALDIPMENUS, and put it somewhere in the MATLAB
%   path. This function should accept one parameter, the MENULIST,
%   and return a modified MENULIST as well as an EXCLUDELIST:
%      [MENULIST,EXCLUDELIST] = LOCALDIPMENUS(MENULIST)
%   The EXCLUDELIST is a cell array with the function names that should
%   not be put into the menus. The MENULIST is a cell array with two
%   columns, the first one containing the names of the sub-menus, the
%   second one containing a cell array with the function names on that
%   menu. Look into the DIPMENUS function for an example.

% (C) Copyright 1999-2003               Pattern Recognition Group
%     All rights reserved               Faculty of Applied Physics
%                                       Delft University of Technology
%                                       Lorentzweg 1
%                                       2628 CJ Delft
%                                       The Netherlands
%
% Cris Luengo & Lucas van Vliet, April 2000.
% 8 August 2001: Added exclude list (finally).
% 16 July 2002: Added a few comments to the documentation inside the function.
% 6 October 2003: Added support for LOCALDIPMENUS.

function [menulist,excludelist] = dipmenus(s)

if nargin ~= 1 | ~strcmp(s,'DIP_GetMenus')
   error('This function does not perform any image processing.')
end

filemenu = {'readim','readcolorim','readroiim','readtimeseries','writeim','sep','readavi','writeavi','writedisplayavi'};
if exist('twain')==3
   filemenu = [filemenu,{'sep','twainselect','acquireim'}];  % optional features
end
if exist('dippreferences')==3 & strcmp(dipgetpref('DebugMode'),'on')
   filemenu = [filemenu,{'sep','reload'}];  % special (internal) command
end
filemenu = [filemenu,{'sep','exit'}];       % special (internal) command: not exiting MATLAB

% The MENU list contains a list of function names for each menu name.
% The string 'sep' inserts a separator between commands.
% The location given here for each function overrides the setting in the
%   function itself (except if it specifies 'none').
menulist = {...
   'File I/O',filemenu;...
   'Display',{'overlay','overlay_label','overlay_confidence','overlay_vector','sep',...
      'orientationplot','sep',...
      'dipgetimage','dipcrop','dipgetcoords','diproi','sep',...
      'diptruesize','dipclf',};...
   'Generation',{'newim','sep','deltaim','ramp','xx','yy','zz','rr','phiphi','sep',...
      'testobject','sep','noise','drawline','drawpolygon','gaussianblob'};...
   'Manipulation',{'shift','rotation','rotation3d','mirror','sep',...
      'resample','subsample'};...
   'Point',{'clip','erfclip','gaussianedgeclip','gaussianlineclip','sep',...
      'stretch','hist_equalize','sep','threshold','sep','lut','get_subpixel'};...
   'Filters',{'convolve','sep','gaussf','gaussf_iir','unif','sep',...
      'maxf','minf','medif','percf','sep','varif','sep',...
      'gabor','gabor_click','sep','canny'};...
   'Differential Filters',{'gauss_derivative','gauss_fourier','sep',...
      'dx','dy','dz','sep','gradmag','gradientvector','sep',...
      'dxx','dyy','dzz','dxy','dxz','dyz','dgg','dcc','sep'...
      'laplace','laplace_plus_dgg','laplace_min_dgg','hessian'};...
   'Adaptive Filters',{'kuwahara','selectionf','sep',...
      'tframehessian','pmd','aniso','mcd','cpf','ced','sep',...
      'gaussf_adap','percf_adap','gaussf_adap_banana','percf_adap_banana','bilateralf'};...
   'Binary Filters',{'bdilation','berosion','bopening','bclosing','hitmiss','sep',...
      'bskeleton','bpropagation','brmedgeobjs','hull','sep',...
      'countneighbours','bmajority',...
      'getsinglepixel','getendpixel','getlinkpixel','getbranchpixel'};...
   'Morphology',{'dilation','erosion','closing','opening','tophat','sep',...
      'dilation_se','erosion_se','closing_se','opening_se','sep',...
      'rankmin_closing','rankmax_opening','rankmin_closing_se','rankmax_opening_se','sep'....
      'reconstruction','sep','watershed'};...
   'Transforms',{'ft','ift','sep','ht','sep','dt','vdt','gdt','sep',...
      'label'};...
   'Analysis',{'measure','msr2obj','msr2ds','measurehelp','sep',...
      'scalespace','morphscales','scale2rgb','sep',...
      'structuretensor','structuretensor3d','curvature','pst','sep',...
      'granulometry','sep','opticflow','findshift','findlocmax',...
      'backgroundoffset','splitandmerge'};...
   'Statistics',{'diphist','mdhistogram','sep','chordlength','paircorrelation','sep',...
      'radialsum','radialmean','radialmin','radialmax','sep',...
      'mse','mre'}...
};

% The EXCLUDE list overrides the MENU list.
% These commands won't appear in the menu, and won't be probed with the 'DIP_GetParamList'
% parameter.
excludelist = {...
   'dip_image_array',...
   'dip_image_new',...
   'newimar',...
   'getparams',...
   'dipfig',...
   'dipgetpref',...
   'dipisosurface',...
   'diplink',...
   'dipmapping',...
   'diporien',...
   'dipsetpref',...
   'dipshow',...
   'dipstep',...
   'diptest',...
   'dipzoom',...
   'jacobi',...
   'orientation4d',...
   'isophote_curvature'...
   'curvature_thirion',...
   'im2array',...
   'array2im',...
   'view5d',...
};

if exist('localdipmenus')
   try
      [menulist,excludelist2] = localdipmenus(menulist);
      excludelist = {excludelist{:},excludelist2{:}};
   catch
      warning('Evaluation of LOCALDIPMENUS failed:')
      disp(lasterr)
   end
end
