%LUT   Look-up Table (with interpolation)
%
% SYNOPSIS:
%  image_out = lut(image_in,table)
%
% PARAMETERS:
%  table: an array with 1 or 3 columns (respectively produce a grey-value
%         image and an RGB image). MATLAB has many commands to create
%         colormaps (see 'help graph3d'). These should be used in this
%         way: lut(image,jet(256)*255)
%
% NOTES:
%  Indexing into the table starts at zero: pixel values should be
%  between 0 and LENGTH(TABLE)-1.
%
%  For integer images, the pixel values are table indices. For floating-
%  point images, the table-lookup is done with interpolation.

% (C) Copyright 1999-2001               Pattern Recognition Group
%     All rights reserved               Faculty of Applied Physics
%                                       Delft University of Technology
%                                       Lorentzweg 1
%                                       2628 CJ Delft
%                                       The Netherlands
%
% Cris Luengo, November 2000.
% June 2001: Fixed bug: integer and floating-point images produced
%            different results.
% December 2001: Added 2D LUTs. The 'table' input must be an image now!
% December 2002: Due to the above change, 3-by-1 tables were converted into 1-by-3.

function image_out = lut(varargin)

d = struct('menu','Point',...
           'display','Look-up table',...
           'inparams',struct('name',       {'image_in',   'table'},...
                             'description',{'Input image','Table'},...
                             'type',       {'image',      'image'},...
                             'dim_check',  {0,            0},...
                             'range_check',{[],           []},...
                             'required',   {1,            1},...
                             'default',    {'a',          []}...
                              ),...
           'outparams',struct('name',{'image_out'},...
                              'description',{'Output image'},...
                              'type',{'image'}...
                              )...
           );
if nargin == 1
   s = varargin{1};
   if ischar(s) & strcmp(s,'DIP_GetParamList')
      image_out = d;
      return
   end
end
% Make sure that the TABLE input argument is not transposed by conversion to
% dip_image and back. This happens only if TABLE is a 1D column.
if nargin == 2
   table = varargin{2};
   if isnumeric(table)
      sz = size(table);
      if sz(2) == 1
         table = dip_image(table);
         varargin{2} = reshape(table,1,sz(1));
      end
   end
end
try
   [image_in,table] = getparams(d,varargin{:});
catch
   if ~isempty(paramerror)
      error(paramerror)
   else
      error(firsterr)
   end
end

out_type = datatype(table);
table = double(table);
if isempty(table)
   error('Table parameter required.');
end
if ~isreal(image_in) | ~isreal(table)
   error('Cannot work on complex data.');
end
[N,w] = size(table);
if w~=1 & w~=3
   if N==1 | N==3
      table = table';
      [N,w] = size(table);
   else
      error('Expecting a LUT with 1 or 3 components');
   end
end
if any(image_in>=N) | any(image_in<0)
   error('Image contains values outside of table.');
end
image_out = newimar(w,1);
for ii=1:w
   switch(class(dip_array(image_in)))
      case {'double','single'}
         image_out{ii} = dip_image(interp1(0:N-1,table(:,ii),double(image_in),'*linear'),out_type);
      otherwise
         try
            t = table(:,ii);
            image_out{ii} = dip_image(t(double(image_in)+1),out_type);
         catch
            error(lasterr);
         end
   end
end
if w==3
   image_out = colorspace(image_out,'RGB');
end
