%MEASURE   Do measurements on objects in an image
%
% SYNOPSIS:
%  msr = measure(object_in,gray_in,measurmentIDs,objectIDs,...
%                 connectivity,minSize,maxSize)
%
% PARAMETERS:
%  object_in: binary or labelled image holding the objects.
%  gray_in: (original) gray value image of object_in. It is needed for
%           several types of measurements. Otherwise you can use [].
%  measurementIDs: measurements to be performed, either a single string
%                  or a cell array with strings (e.g.: {'Size','Perimeter'} ).
%                  See MEASUREHELP for a full list of possible
%                  measurements.
%  objectIDs: labels of objects to be measured. Use [] to measure all
%             objects
%  connectivity: defines which pixels are considered neighbours: up to
%     'connectivity' coordinates can differ by maximally 1. Thus:
%     * A connectivity of 1 indicates 4-connected neighbours in a 2D image
%       and 6-connected in 3D.
%     * A connectivity of 2 indicates 8-connected neighbourhood in 2D, and
%       18 in 3D.
%     * A connectivity of 3 indicates a 26-connected neighbourhood in 3D.
%     Connectivity can never be larger than the image dimensionality.
%  minSize, maxSize: minimum and maximum size of objects to be measured.
%
% DEFAULTS:
%  measurementIDs = 'size'
%  objectIDs = []
%  connectivity = 1
%  minSize = 0
%  maxSize = 0
%
% RETURNS:
%  msr: a dip_measurement object containing the results.
%
% EXAMPLE:
%  img = readim('cermet')
%  msr = measure(img<100, img, ({'size', 'perimeter','mean'}), [], ...
%                1, 1000, 0)
%
% NOTE:
%  The function MEASUREHELP provides help on the measurement
%  features available in this function.

% (C) Copyright 1999-2004               Pattern Recognition Group
%     All rights reserved               Faculty of Applied Physics
%                                       Delft University of Technology
%                                       Lorentzweg 1
%                                       2628 CJ Delft
%                                       The Netherlands
%
% Cris Luengo, August 2000.
% September 2000: Added 'optionarray' for measurementIDs.
% September 2000: Added MINSIZE and MAXSIZE parameters.
% December 2000:  Fixed bug when using MINSIZE or MAXSIZE with new dip_measurement.
% January 2002:   Returning empty structure if MINSIZE is larger than any object.
% February 2002:  Using the new function 'dip_getmeasurefeatures' - removed static lists.
% February 2004:  Improved help (KvW).
% February 2004:  Fixed bug involving min and max size. Default measurement is 'size'.

function data = measure(varargin)

msmts = dip_getmeasurefeatures;
% Remove private element from list - This is Michael's useless creation.
I = find(strcmpi({msmts.name},'BendingEnergy'));
if length(I)==1
   msmts(I) = [];
end

d = struct('menu','Analysis',...
           'display','Measure',...
           'inparams',struct('name',       {'object_in',   'gray_in',         'measurementID','objectIDs', 'connectivity','minSize',            'maxSize'},...
                             'description',{'Object image','Grey-value image','Measurement',  'Object IDs','Connectivity','Minumum object size','Maximum object size'},...
                             'type',       {'image',       'image',           'optionarray',  'array',     'array',       'array',              'array'},...
                             'dim_check',  {0,             0,                 1,              -1,          0,             0,                    0},...
                             'range_check',{[],            [],                msmts,          'N+',        'N+',          'N',                  'N'},...
                             'required',   {1,             0,                 0,              0,           0,             0,                    0},...
                             'default',    {'a',           '[]',              'size',  [],          1,             0,                    0}...
                            ),...
           'outparams',struct('name',{'msr'},...
                              'description',{'Output measurement data'},...
                              'type',{'measurement'}...
                              )...
          );
if nargin == 1
   s = varargin{1};
   if ischar(s) & strcmp(s,'DIP_GetParamList')
      data = d;
      return
   end
end
%%% aliases for elements in the 'msmts' list (backwards compatability).
if nargin>=3
   if ischar(varargin{3})
      varargin{3} = {varargin{3}};
   end
   if iscellstr(varargin{3})
      for ii=1:prod(size(varargin{3}))
         varargin{3}{ii} = mapaliases(varargin{3}{ii});
      end
   end
end
%%%
try
   [object_in,gray_in,measurementID,objectIDs,connectivity,minSize,maxSize] = getparams(d,varargin{:});
catch
   if ~isempty(paramerror)
      error(paramerror)
   else
      error(firsterr)
   end
end

if islogical(object_in)
   % meaning it is a binary image...
   object_in = dip_label(object_in,connectivity,'threshold_on_size',...
                         minSize,maxSize,'');
   if max(object_in) < 1
      data = dip_measurement;
      return
   end
elseif minSize ~= 0 | maxSize ~= 0
   % measure object size and select objects in range
   % We don't need to do this if we just labelled the image
   if minSize ~= 0 & maxSize ~= 0 & minSize > maxSize
      error('maxSize must be larger than minSize.')
   end
   sz = dip_measure(object_in,gray_in,'size',objectIDs,connectivity);
   if minSize ~= 0
      sz = sz(sz.id(sz.size>=minSize));
   end
   if maxSize ~= 0
      sz = sz(sz.id(sz.size<=maxSize));
   end
   objectIDs = sz.id;
   if length(objectIDs) < 1
      data = dip_measurement;
      return
   end
elseif max(object_in) < 1
   data = dip_measurement;
   return
end
data = dip_measure(object_in,gray_in,measurementID,objectIDs,connectivity);


%%%

function feature = mapaliases (feature)
   switch lower(feature)
      case 'dimension'
         feature = 'Dimensions';
      case 'min'
         feature = 'Minimum';
      case 'max'
         feature = 'Maximum';
      case 'surfarea'
         feature = 'SurfaceArea';
      case 'ginertia'
         feature = 'GreyInertia';
      case 'gmu'
         feature = 'GreyMu';
      case 'dominantorientation2d'
         feature = 'Orientation2D';
   end
