%PMD   Perona Malik anisotropic diffusion
%
% SYNOPSIS:
%  outimage = anisodiff(in, iterations, K, lambda)
%
% PARAMETERS:
%  in = 2D or 3D image with equal sampling rates in all dimensions
%  K = edge-stopping parameter (4% of the image's range is a good start)
%  lambda = diffusion step (<1, smaller + more iterations = more accurate)
%
% DEFAULTS:
%  iterations = 5
%  K = 10 
%  lambda = 0.25
%
% EXAMPLE:
%  pmd(readim,5,10)
%  pmd(readim('nr077_01'),5,40)
%
% EXAMPLE:
%  Edge stopping effect: view imaginary part of ans with log stretch
%  you would see the kernel evolution of initial pixels in d, they diffuse 
%  but stop @ edge. Yet, since the pixels are so close to the edge
%  their diffusion is slow. for pix right on edge as in f, it is clear
%  that diffusion occur along local orientation, edge still avoided
%  a = readim
%  b = gradmag(a)
%  c = threshold(b)
%  d = xor(bdilation(c),c) & mod(xx,10)==0      % pix close to edge
%  e = xor(bdilation(c,2),bdilation(c,1)) & mod(xx,10)==0   % further from edge
%  f = bskeleton(berosion(c))       % (mostly) single pix on edge
%  pmd(a+d*i,5,10)
%
% LITERATURE: 
%  Perona, Shiota and Malik, "Anisotropic Diffusion", in
%   B.M. ter Haar Romeny (Ed.), "Geometry-Driven Diffusion in Computer Vision", 1994
%
% SEE ALSO: mcd, mcm, ced


% (C) Copyright 1999-2003               Pattern Recognition Group
%     All rights reserved               Faculty of Applied Physics
%                                       Delft University of Technology
%                                       Lorentzweg 1
%                                       2628 CJ Delft
%                                       The Netherlands
%
% Tuan Pham, August 2002
% November 2003, added 3D support (TP)

function out = pmd(varargin)

d = struct('menu','Adaptive Filters',...
	'display','Perona-Malik Diffusion',...
	'inparams',struct('name',{'in','iterations','K','lambda'},...
   'description',{'Input image','Iterations','K','lambda'},...
   'type', 		{'image','array','array','array'},...
   'dim_check',  {0,0,0,1},...
   'range_check',{[],'R+','R+','R+'},...
   'required',	{1,0,0,0},...
   'default', 	{'a',5,10,0.25}...
	),...
   'outparams',struct('name',{'out'},...
   	  'description',{'Output'},...
   	  'type',{'image'}...
   	  )...
 );

if nargin == 1
   s = varargin{1};
   if ischar(s) & strcmp(s,'DIP_GetParamList')
      out = d;
      return
   end
end
try
   [in,iterations,K,lambda] = getparams(d,varargin{:});
catch
   if ~isempty(paramerror)
      error(paramerror)
   else
      error(firsterr)
   end
end

if ndims(in)>3
   error('Only supporting <4D images up to now.');
end 

% old function declaration & default parameters
%function [outimage] = anisodiff(in, iterations, K, lambda)
%if nargin<2|isempty(iterations) iterations = 5, end;
%if nargin<3|isempty(K) K = (max(in)-min(in))/25, end;
%if nargin<4|isempty(lambda) lambda = 0.25, end;

out = in;
[m,n,p] = size(in);
rowC = [0:m-1];
colC = [0:n-1];
rowN = [0 0:m-2];
colE = [0 0:n-2];
rowS = [1:m-1 m-1];
colW = [1:n-1 n-1];

if length(size(in))==2   % original Perona-Malik code
 for ii=1:iterations,
   deltaN = out(rowN,colC) - out(rowC,colC);
   deltaE = out(rowC,colE) - out(rowC,colC);

   % since there is an abs operation here, small imaginary part 
   % do not polute the result like original tukeyPsi in rad.m
   fluxN = deltaN .* exp( - (1/K) * abs(deltaN) );       
   fluxE = deltaE .* exp( - (1/K) * abs(deltaE) );
   
   out = out + lambda(1)*(fluxN - fluxN(rowS,colC)) + lambda(2)*(fluxE - fluxE(rowC,colW));
 end;
 
elseif length(size(in))==3  % extension of Perona-Malik to 3D
 heiC = [0:p-1];        % height Center
 heiF = [0 0:p-2];      % height Front
 heiB = [1:p-1 p-1];    % height Back
 for ii=1:iterations,   % different execution order to save memory
   deltaN = out(rowN,colC,heiC) - out(rowC,colC,heiC);
   fluxN = deltaN .* exp( - (1/K) * abs(deltaN) ); clear deltaN;
   out = out + lambda(1) * (fluxN - fluxN(rowS,colC,heiC)); clear fluxN;
   
   deltaE = out(rowC,colE,heiC) - out(rowC,colC,heiC);
   fluxE = deltaE .* exp( - (1/K) * abs(deltaE) ); clear deltaE;
   out = out + lambda(2) * (fluxE - fluxE(rowC,colW,heiC)); clear fluxE;
   
   deltaF = out(rowC,colC,heiF) - out(rowC,colC,heiC);
   fluxF = deltaF .* exp( - (1/K) * abs(deltaF) ); clear deltaF;
   out = out + lambda(3) * (fluxF - fluxF(rowC,colC,heiB)); clear fluxF;
 end;

end;
 

