%READCOLORIM   Read color image from file
%
% SYNOPSIS:
%  image_out = readcolorim(filename,format)
%
% PARAMETERS:
%  filename: string with name of file, optionally with path and extension.
%  format: string with any of: 'ICS', 'TIFF', 'GIF', 'PIC' or 'CSV', or an empty
%          string, which will cause the function to search for the correct
%          type.
%
% DEFAULTS:
%  filename = 'flamingo.tif'
%  format = ''
%
% NOTES:
%  To get a current list of file formats, use the function DIPIO_GETIMAGEREADFORMATS,
%  which also gives a short description of each format.
%
%  For JPEG, BMP, PNG and other such files not recognized by dipIO, leave
%  FORMAT empty. This will allow READIM to call IMREAD (a MATLAB function)
%  that does recognize these files.

% (C) Copyright 1999-2003               Pattern Recognition Group
%     All rights reserved               Faculty of Applied Physics
%                                       Delft University of Technology
%                                       Lorentzweg 1
%                                       2628 CJ Delft
%                                       The Netherlands
%
% Cris Luengo, July 2000.
% 8 October 2000: Calling COLORSPACE now.
% 17 December 2001: Using MATLAB's IMREAD if dipIO cannot read the file.
% 22 March 2002: Using new JOINCHANNELS syntax.
% 16 July 2002: Using new DIPIO_IMAGEREADCOLOR function.
% 16 September 2003: Fixed bug reading indexed color images through MATLAB.

function image_out = readcolorim(varargin)

frmts = [struct('name','','description','Any type'),dipio_getimagereadformats];

d = struct('menu','File I/O',...
           'display','Read color image',...
           'inparams',struct('name',       {'filename',                'format'},...
                             'description',{'Name of the file to open','File format'},...
                             'type',       {'infile',                  'option'},...
                             'dim_check',  {0,                         0},...
                             'range_check',{'*.*',                     frmts},...
                             'required',   {0,                         0},...
                             'default',    {'flamingo.tif',            ''}...
                            ),...
           'outparams',struct('name',{'image_out'},...
                              'description',{'Output image'},...
                              'type',{'image'}...
                              )...
          );
if nargin == 1
   s = varargin{1};
   if ischar(s) & strcmp(s,'DIP_GetParamList')
      image_out = d;
      return
   end
end
%%% aliases for elements in the 'format' list.
if nargin>=2 & ischar(varargin{2})
   if strcmpi(varargin{2},'tif')
      varargin{2} = 'tiff';
   end
end
%%%
try
   [filename,format] = getparams(d,varargin{:});
catch
   if ~isempty(paramerror)
      error(paramerror)
   else
      error(firsterr)
   end
end

try
   [image_out,photometric] = dipio_imagereadcolour(filename,format,1);
catch
   error(firsterr)
end
if isempty(image_out) & isempty(fileparts(filename)) % The file name has no path.
   p = getimagefilepath;
   for ii=1:length(p)
      fname = fullfile(p{ii},filename);
      try
         [image_out,photometric] = dipio_imagereadcolour(fname,format,1);
      catch
         error(firsterr)
      end
      if ~isempty(image_out)
         break;
      end
   end
end
if isempty(image_out)
   if ~isempty(format)
      error(['File not found or type not recognized (leave ''format'' empty to allow IMREAD to be called).']')
   end
   try
      [image_out,map] = imread(filename);
   catch
      error('File not found or type not recognized.')
   end
   if ~isempty(map)
      image_out = lut(image_out,map*255);
   elseif ndims(image_out)==3
      if isa(image_out,'double')
         image_out = image_out*255;
      end
      image_out = dip_image(image_out);
      image_out = joinchannels('RGB',image_out);
   else
      image_out = dip_image(image_out);
   end
else
   if ~strcmp(photometric,'gray')
      image_out = joinchannels(photometric,image_out);
   end
end
