%READROIIM   Read ROI of a grey-value image from file
%
% SYNOPSIS:
%  out = readroiim(filename, offset, roi, sampling, format)
%
% PARAMETERS:
%  filename: string with name of file, optionally with path and extension.
%  offset: integer array
%  roi: integer array
%  sampling: integer arry
%  format: string with any of: 'ICS', 'PIC' or an empty
%          string, which will cause the function to search for the correct
%          type.
%
% DEFAULTS:
%  filename = 'erica.ics'
%  sampling = 1
%  format = ''
%
% EXAMPLE:
%  out = readroiim('erika',[64 64],[128 128],[2 2])
%
% This function is useful for large files, where only a smaller region is
% needed for processing


% (C) Copyright 1999-2002               Pattern Recognition Group
%     All rights reserved               Faculty of Applied Physics
%                                       Delft University of Technology
%                                       Lorentzweg 1
%                                       2628 CJ Delft
%                                       The Netherlands
%
% Sep 2002, Cris Luengo, Bernd Rieger


function image_out = readroiim(varargin)

frmts = dipio_getimagereadformats;
ICSi = find(strcmp({frmts.name},'ICS'));
PICi = find(strcmp({frmts.name},'PIC'));
frmts = struct('name',{'','ICS','PIC'},'description',{'Any type',frmts(ICSi).description,frmts(PICi).description});

d = struct('menu','File I/O',...
   'display','Read ROI image',...
   'inparams',struct('name',{'filename','offset','roi','samp','format'},...
     'description',{'Name of the file to open','Offset','ROI','Sampling','File format'},...
     'type',       {'infile','array','array','array','option'},...
     'dim_check',  {0,-1,-1,-1,0},...
     'range_check',{'*.*','R+','R+','R+', frmts},...
     'required',   {0,1,1,1,0},...
     'default',    {'erika.ics',[],[],[],''}...
    ),...
   'outparams',struct('name',{'image_out'},...
                      'description',{'Output image'},...
                      'type',{'image'}...
                      )...
  );
if nargin == 1
   s = varargin{1};
   if ischar(s) & strcmp(s,'DIP_GetParamList')
      image_out = d;
      return
   end
end

try
   [filename,offset,roi,samp,format] = getparams(d,varargin{:});
catch
   if ~isempty(paramerror)
      error(paramerror)
   else
      error(firsterr)
   end
end

try
   image_out = dipio_imagereadroi(filename,offset,roi,samp,format,1);
catch
   error(firsterr)
end
if isempty(image_out) & isempty(fileparts(filename)) % The file name has no path.
   p = getimagefilepath;
   for ii=1:length(p)
      fname = fullfile(p{ii},filename);
      try
         image_out = dipio_imagereadroi(fname,offset,roi,samp,format,1);
      catch
         error(firsterr)
      end
      if ~isempty(image_out)
         break;
      end
   end
end
if isempty(image_out)
   if ~isempty(format)
      error(['File not found or type not recognized (leave ''format'' empty to allow IMREAD to be called).']')
   end
   try
      [image_out,map] = imread(filename);
   catch
      error('File not found or type not recognized.')
   end
end
