%READTIMESERIES   Reads a series of images from disk in one image. 
%
% SYNOPSIS 
%  out = readtimeseries(basefilename, extension, range, color)
%   
%  basefilename : File name without the running number
%  extension    : File extension 
%  range        : first and last index to read [first last]
%  color        : Do the files contain color information 'no','yes'
%
% DEFAULTS:
%  extension    : ics
%  range        : [0 0] %all found images are read
%  color        : no 
%
% EXAMPLE:
%  In the directory /data/here, there are 16 files:
%   myfile4.tif,myfile5.tif,myfile006.tif,...myfile020.tif
%  out = readtimeseries('/data/here/myfile','tif')
%  
% SEE ALSO:
%  readim, readcolorim 
%
% NOTE:
%  For files like myleica_z004_ch01.tif etc. use 
%  readtimeseries('myleica_z*_ch01','tif')

% (C) Copyright 1999-2004               Pattern Recognition Group
%     All rights reserved               Faculty of Applied Physics
%                                       Delft University of Technology
%                                       Lorentzweg 1
%                                       2628 CJ Delft
%                                       The Netherlands
% Bernd Rieger, June 2001.
% March, 2004, renamed D2toD3 to readtimeseries and added more functionality (BR)
% April, 2004, changed filelist to be more flexiable, use now find_files 
%              instead of num_slices (BR)
% April, 2004, allowing empty range parameter (I'm lazy!). Removed second reading
%              of first input image (Bernd is lazy too!). Provided for the (unlikely)
%              case that the input slices are not RGB. (CL)

function out=readtimeseries(varargin)

d = struct('menu','File I/O',...
 'display','Read Time Series',...
 'inparams',struct('name',       {'in','ext','range','col'},...
          'description',{'Filename (base)','Extension','Range','Color information'},...
          'type',       {'string','option','array','boolean'},...
          'dim_check',  {0,0,-1,0},...
          'range_check',{'*.*',{'ics','tif','csv','pic','gif'},'N',[]},...
          'required',   {1,0,0,0},...
          'default',    {'film1','ics',[0 0],0}...
           ),...
 'outparams',struct('name',{'out'},...
                    'description',{'Output image'},...
                    'type',{'image'}...
                    )...
 );
if nargin == 1
   s = varargin{1};
   if ischar(s) & strcmp(s,'DIP_GetParamList')
      out = d;
      return
   end
end
try
   [in,ext,range,col] = getparams(d,varargin{:});
catch
   if ~isempty(paramerror)
      error(paramerror)
   else
      error(firsterr)
   end
end
if isempty(range)
   range = [0,0];
elseif prod(size(range))~=2
   error('The range parameter must be of the form [start ending]');
end

fns = find_files(in,ext,range(1),range(2));
M = length(fns);
if M == 0
   error('Found no matching files.');
end

tmp = squeeze(readim(fns{1},ext)); % this also works for color images to get the size, dims
fdim = length(size(tmp));
fprintf('Reading %dD file: %s\n',fdim,fns{1});

colsp = '';
if col
   tmp = readcolorim(fns{1});
   out = newimar(size(tmp));
   colsp = colorspace(tmp);
   for ii = 1:prod(size(tmp))
      out{ii} = newim([size(tmp{ii}) M],datatype(tmp{ii}));
   end
   ss = ['out(' repmat(':,',1,fdim) 'ii-1) = readcolorim(fns{ii},ext);'];
else
   out = newim([size(tmp) M],datatype(tmp));
   ss = ['out(' repmat(':,',1,fdim) 'ii-1) = readim(fns{ii},ext);'];
end
%ii = 1
%eval (ss)
eval (['out(' repmat(':,',1,fdim) '0) = tmp;'])

for ii = 2:M
   %s= [in sprintf(sz0,ii) '.' ext];
   fprintf('Reading %dD file: %s\n',fdim,fns{ii});
   eval(ss)
end
if ~isempty(colsp)
   out = colorspace(out,colsp);
end
