%THRESHOLD   Thresholding
%
% SYNOPSIS:
%  [out, th_value] = threshold(in,type,parameter)
%
% PARAMETERS:
%  type: string, one of:
%     'isodata':    Thresholding using the Isodata algorithm,
%                   for more options (mask image, several thresholds)
%                   see dip_isodatathreshold.
%     'triangle':   Thresholding using chord method
%                   (a.k.a. skewed bi-modality, maximum distance to triangle)
%                   by Zack, Rogers and Latt (1973).
%     'background': Thresholding using unimodal background-symmetry method.
%     'fixed':      Thresholding at a fixed value.
%     'double':     Thresholding between two fixed values.
%     'volume':     Thresholding to obtain a given volume fraction.
%     'hysteresis':  From the binary image (in>low) only those regions are
%                   selected for which at least one pixel is (in>high)
%
%  parameter ('background'): Distance to the peak where we cut-off, in
%                            terms of the half-width at half the maximum.
%                            Inf selects the default value, which is 2.
%            ('fixed'):      Threshold value. Inf means halfway between
%                            minimum and maximum value.
%            ('double'):     Two threshold values. Inf means
%                            min+[1/3,2/3]*(max-min).
%            ('volume'):     Parameter = the volume fraction (Inf means 0.5)
%            ('hysteresis'): Two values: low, high (see above)
% 
% DEFAULT:
%  type:      'isodata'
%  parameter: Inf (means: use default for method)
% 
% See Also DIP_RANGETHRESHOLD, DIP_HYSTERESISTHRESHOLD


% (C) Copyright 1999-2004               Pattern Recognition Group
%     All rights reserved               Faculty of Applied Physics
%                                       Delft University of Technology
%                                       Lorentzweg 1
%                                       2628 CJ Delft
%                                       The Netherlands
% Bernd Rieger, May 2001.
% with code by Lucas van Vliet and Cris Luengo
% 31 August    2001 (CL):  Added 'double' method after comments by Geert.
% 11 September 2001 (CL):  'double' method now uses dip_rangethreshold.
% 14 September 2001 (CL):  Fixed 'background' method. Vectorized 'triangle' method.
% 25 September 2001 (CL):  Fixed small bug in 'triangle' and 'background' methods.
%  8 March     2002 (CL):  Catching dfloat bug in dip_histogram in a better way.
%  9 April     2002 (MvG): Replacing dip_histogram() by diphist().
%  9 October   2002 (MvG): Added "kmeans" alias for "isodata".
% 20 October   2002 (MvG): Added "volume" method.
% 23 March     2004 (BR):  Integrated "hystersis" method from KvW 
%                          return threshold value also in menu.
% 01 April     2004 (KvW): Changed "hystersis" to "hysteresis"
%                          Threshold value high can be lower than threshold value low
%                          which will yield the same result as fixed thresholding.
%  8 April     2004 (CL):  Added check for binary or complex input image.
 
function [varargout] = threshold(varargin)

d = struct('menu','Point',...
           'display','Threshold',...
           'inparams',struct('name',       {'in','type','parameter'},...
                  'description',{'Input image','Type','Parameter'},...
                  'type',       {'image','option','array'},...
                  'dim_check',  {0,0,-1},...         
'range_check',{[],{'isodata','triangle','background','fixed','double','volume','hysteresis'},[]},...
                  'required',   {1,0,0},...
                  'default',    {'a','isodata',Inf}...
                   ),...
           'outparams',struct('name',{'out','threshold_new'},...
                   'description',{'Output image','Threshold value'},...
                   'type',{'image','array'}...
                   )...
           );
if nargin == 1
   s = varargin{1};
   if ischar(s) & strcmp(s,'DIP_GetParamList')
      varargout{1} = d;
      return
   end
end
%%% aliases for elements in the 'type' list.
if nargin>=2 & ischar(varargin{2})
   if strcmpi(varargin{2},'kmeans')
      varargin{2} = 'isodata';
   end
end

try
   [in,type,parameter] = getparams(d,varargin{:});
catch
   if ~isempty(paramerror)
      error(paramerror)
   else
      error(firsterr)
   end
end

if islogical(in)
   error('Cannot threshold a binary image.')
elseif ~isreal(in)
   error('Cannot threshold a complex-valued image.')
end

switch type

   case 'isodata'
      if strcmp(datatype(in),'dfloat')
         in = dip_image(in,'sfloat');
      end
      [varargout{1}, varargout{2}] = dip_isodatathreshold(in,[],1);
      varargout{1} = dip_image(varargout{1},'bin');

      % Strange cris code that does something completly different. (BR)
      % No, it works. And Lucas wrote it. (CL)
      %min_val = min(in);
      %max_val = max(in);
      %range = max_val-min_val;
      %interval = (max_val-min_val)/199;
      %[histogram,bins] = dip_histogram(in,[],interval,max_val,min_val,0);
      %histogram = double(histogram);
      %threshold_old = min_val;
      %threshold_new = min_val + range/2;
      %while (threshold_new - threshold_old < interval/10)
      %   mask = (bins < threshold_new);
      %   mean1 = sum(bins(mask).*histogram(mask)) / sum(histogram(mask));
      %   mask = (bins >= threshold_new);
      %   mean2 = sum(bins(mask).*histogram(mask)) / sum(histogram(mask));
      %   threshold_old = threshold_new;
      %   threshold_new = (mean1 + mean2) / 2;
      %end

   case 'triangle'

      % A smooth histogram
      min_val = min(in);
      max_val = max(in);
      interval = (max_val-min_val)/199;
      border = 16;
      max_val = max_val+border*interval;
      min_val = min_val-border*interval;
      [histogram,bins] = diphist(in,[min_val,max_val],200);
      histogram = gaussf(histogram,4);
      histogram = double(histogram);
      bins = double(bins);

      % Find peak
      [max_value,max_element] = max(histogram);

      % Define: start, peak, stop positions in histogram
      sz = length(histogram);
      left_bin  = [border,histogram(border)];
      right_bin = [sz-border,histogram(sz-border)];
      top_bin   = [max_element,max_value];

      % Find the location of the maximum distance to the triangle
      v1 = top_bin - left_bin;
      if any(v1~=0)
         v1 = v1/norm(v1);
      end
      ii = left_bin(1):top_bin(1);
      v2 = [ii'-left_bin(1),histogram(ii)'-left_bin(2)];
      distance = abs(v1(1).*v2(:,2)-v1(2).*v2(:,1));
      v1 = top_bin - right_bin;
      if any(v1~=0)
         v1 = v1/norm(v1);
      end
      jj = top_bin(1):right_bin(1);
      v2 = [jj'-right_bin(1),histogram(jj)'-right_bin(2)];
      distance = [distance;abs(v1(1).*v2(:,2)-v1(2).*v2(:,1))];
      [max_distance,bin] = max(distance);
      ii = [ii,jj];
      bin = ii(bin);

      % Lucas' old code:
      %bin = 0;
      %v1 = top_bin - left_bin;
      %if v1 ~= 0
      %   v1 = v1 / sqrt(v1*v1'); % Normalize the vector
      %end
      %max_distance = 0;
      %for ii = left_bin(1):top_bin(1)
      %   pos = [ii, histogram(ii)];
      %   v2 = pos - left_bin;
      %   D = [v1; v2];
      %   distance = sqrt(det(D*D'));
      %   if distance > max_distance;
      %      max_distance = distance;
      %      bin = ii;
      %   end
      %end
      %v1 = top_bin - right_bin;
      %v1 = v1 / sqrt(v1*v1'); % Normalize the vector
      %for ii = top_bin(1):right_bin(1)
      %   pos = [ii, histogram(ii)];
      %   v2 = pos - right_bin;
      %   D = [v1; v2];
      %   distance = sqrt(det(D*D'));
      %   if distance > max_distance
      %      max_distance = distance;
      %      bin = ii;
      %   end
      %end

      varargout{2} = bins(bin);
      varargout{1} = in >= varargout{2};

   case 'background'

      if length(parameter)~=1 | parameter<=0
         error('Positive scalar value expected as parameter for ''background'' method.')
      end
      if ~isfinite(parameter)
         parameter = 2;        % Default value
      end

      % A smooth histogram
      min_val = min(in);
      max_val = max(in);
      interval = (max_val-min_val)/199;
      border = 16;
      max_val = max_val+border*interval;
      min_val = min_val-border*interval;
      [histogram,bins] = diphist(in,[min_val,max_val],200);
      histogram = gaussf(histogram,4);
      histogram = double(histogram);
      bins = double(bins);

      % Find peak
      [max_value,max_element] = max(histogram);

      % Is the peak on the left or right side of the histogram?
      cumhist = cumsum(histogram);
      cumhist = cumhist/cumhist(end);
      rightpeak = cumhist(max_element) > 0.5;
      % Find the 50% height & the theshold
      if rightpeak
         sigma = find(histogram(max_element:end) < (max_value/2));
         if length(sigma) ~= 0
            sigma = sigma(1) - 1;
         else
            sigma = 1;
         end
         varargout{2} = bins(max_element - sigma*parameter);
      else
         sigma = find(histogram(1:max_element) < (max_value/2));
         if length(sigma) ~= 0
            sigma = max_element - sigma(end);
         else
            sigma = 1;
         end
         varargout{2} = bins(max_element + sigma*parameter);
      end

      varargout{1} = in >= varargout{2};

   case 'fixed'
      if length(parameter)~=1
         error('Scalar value expected as parameter for ''fixed'' method.')
      end
      varargout{2} = parameter;
      if ~isfinite(varargout{2})
         varargout{2} = (max(in)+min(in))/2;
      end
      varargout{1} = in >= varargout{2};

   case 'double'
      if any(~isfinite(parameter))
         mn = min(in);
         threshold_new = mn + [1/3,2/3]*(max(in)-mn);
      elseif length(parameter)~=2
         error('Two values expected as parameter for ''double'' method.')
      else
         threshold_new = parameter;
         if threshold_new(1) > threshold_new(2)
            threshold_new = threshold_new([2,1]);
         end
      end
      varargout{1} = dip_rangethreshold(in,threshold_new(1),threshold_new(2),1,0,1);
      varargout{2} = 0;
      
   case 'volume'
      fraction = 0.5;
      if length(parameter)>0
         if isfinite(parameter(1))
            fraction = parameter(1);
         end
         if fraction<0 | fraction>1
            error('Volume fraction must be in range [0;1]');
         end
      end
      if length(parameter)>1
         error('Volume fraction method has exactly one parameter');
      end
      varargout{2}=percentile(in, 100*(1-fraction));
      varargout{1} = in >= varargout{2};

   case 'hysteresis'
      if length(parameter)~=2
         error('Two values expected as parameter for ''hystersis'' method.')
      end
      tmp = parameter;
%      if tmp(1)>tmp(2)
%         tmp = tmp([2 1]);
%      end
      varargout{1} = dip_hysteresisthreshold(in,tmp(1),tmp(2));
      varargout{2} = 0;
   otherwise
      error('Unknown threshold method.');
end
