%VIEW5D Start the java viewer by Rainer Heintzmann
%
% VIEW5D(in,ts,extern) starts a java based image viewer
%
% PARAMETERS:
%   in:     can either be a dip_image or a figure handle
%   ts:     is the input a time series? (0,1)
%   extern: start the extern viewer, if no java support 
%           within matlab. Writes a file to the tempdir (0,1) 
%
% DEFAULTS:
%  ts:     0
%  extern: 0
%
% See also DIPSHOW.
%
% NOTE:
%  The path of java archive View5D.jar must be in the classpath.txt file 
%  For the extern viewer View5D.jar must be present 
%  (symbolic link is not sufficient) in the OS tempdir and the 
%  java appletviewer installed.
%
%  For larger images you may get java.lang.OutOfMemoryError
%  Increase the memory for the jvm by placing the file java.opts in your
%  MATLAB directory with -Xmx#bytes (java 1.3) or maxHeapSize=#bytes (older)
%  Set #bytes to something large (600000000).
%
% More information about the viewer
%   www.gwdg.de/~rheintz/View5D


% (C) Copyright 1999-2004               Pattern Recognition Group
%     All rights reserved               Faculty of Applied Physics
%                                       Delft University of Technology
%                                       Lorentzweg 1
%                                       2628 CJ Delft
%                                       The Netherlands
%
% Bernd Rieger
function out = view5d(varargin)
% Parse input
N = nargin;
if N == 1
   arg = varargin{1};
   if ischar(arg) & strcmp(arg,'DIP_GetParamList')
      out = struct('menu','none');
      return
   end
   ts  =0;
   extern =0;   
elseif N >3
   error('Too many input arguments.')
end

in = varargin{1};


if isa(in,'dip_image') | isa(in,'dip_image_array')
   if N ==1 & java
      out=view5d_image_direct(in,ts);      
   elseif N ==2 & java      
      out=view5d_image_direct(in,varargin{2});
   else
      fprintf('Extern viewer, matlab started without java support.\n');      
      view5d_image_extern(in,varargin{2});
   end
else
   if N ==2
      ts = varargin{2};
   else
      ts =0;
   end
   out=view5d_window(in,ts);   
end

%------------------------------------------------------------------
function out=view5d_window(in,ts)
udata = get(in,'UserData');
if isfield(udata,'channels')
   %color images
   elements = udata.channels;  
else
   elements = 1;
end
sz = udata.imsize;
if length(sz)==2
   sz(3)=1;
   tmp = udata.imagedata;
else
   if elements >1 % 3D color image
      tmp = array2im(udata.slices);
   else
      tmp = udata.slices;      
   end  
end
if ts
   ts = sz(end);
else
   ts =1;
end

in5d = dip_array(reshape(tmp,1,prod(size(tmp))));
%size(in5d)
%whos in5d
%elements
%ts
out=View5D.Start5DViewer(in5d,sz(1),sz(2),sz(3),elements,ts);


%------------------------------------------------------------------
function out=view5d_image_direct(in,ts)

if isa(in,'dip_image_array')
   if ~isvector(in)
      error('No vector/spectral image.');
   end
   elements = prod(size(in));
   sz = size(in{1}); 
   dt = datatype(in{1});  
   in = array2im(in);
else
   sz = size(in);
   elements = 1;
   dt = datatype(in);
end
if length(sz)>5 | length(sz)<2
   error('Only available for 2,3,4D images.');
end
if length(sz)==2
   sz(3)=1;
end
if ts
   ts = sz(end);
else
   ts =1;
end

%Make a one dimensional flat input array
in5d = dip_array(reshape(in,1,prod(size(in))));

size(in5d)
whos in5d
%sz
%elements
%ts
out=View5D.Start5DViewer(in5d,sz(1),sz(2),sz(3),elements,ts);

%-----------------------------------------------------------------------
function view5d_image_extern(in,ts)
%needs the appletviewer to exist and the View5D.jar in the tempdir
%this is still TODO, if the location of View5D.jar is standart   


if iscolor(in)
   sz = size(in{1});
   elements = 3;   
   dt = datatype(in{1});
elseif isa(in,'dip_image_array')
   if ~isvector(in)
      error('No vector/spectral image.');
   end
   elements = prod(size(in));
   sz = size(in{1}); 
   dt = datatype(in{1});  
   in = array2im(in);
else
   sz = size(in);
   elements = 1;
   dt = datatype(in);
end
if length(sz)>5 | length(sz)<2
   error('Only available for 2,3,4D images.');
end
if length(sz)==2
   sz(3)=1;
end
if ts
   ts = sz(end);
else
   ts =1;
end
mdt = {'uint8',  '1',  '8',...
       'uint16', '2',  '16',...
       'uint32', '4',  '32',...
       'sint8',  '1',  '8',...
       'sint16', '2',  '16',...
       'sint32', '4',  '32',...
       'sfloat', '-1', '32',...
       'dfloat', '8',  '64',...
       'bin',    '1',  '8'};
ind = strmatch(dt,mdt);
bytes = str2num(char(mdt(ind+1)));
bits  = str2num(char(mdt(ind+2)));

base = tempdir;%return os set tempdir
if ~tempdir
   error('No temp directory given by OS.');
end
fn1 = 'dipimage_view5d';
fn = [base fn1];

fid = fopen([base 'View5D.jar']);
if fid <0
   error('Could not find the java archive %s\n',[base 'View5D.jar']);
end

%write raw data
writeim(in,fn,'icsv1',0);
s2 = system(['rm ' fn '.ics']);

%'<param name=ts value=' num2str(tt) ' >\n'...
% write html file
content = ['<html><head>\n'...
      '<title>5D Viewer</title></head><body>\n'...
      '<h1>5D Viewer</h1>Java-Applet by Rainer Heintzmann\n' ...
      '<p>Image Data displayed by View5D<hr>\n' ...
      'please wait for image data to be loaded<p>\n' ...
      '<p>Automatically generated by DIPimage.</p>\n' ...
      '<applet archive=View5D.jar code=View5D width=600 height=700>\n'...
      '<param name=file value=file:' fn '.ids>\n'...
      '<param name=sizex value=' num2str(sz(1)) ' >\n'...
      '<param name=sizey value=' num2str(sz(2)) ' >\n'...
      '<param name=sizez value=' num2str(sz(3)) ' >\n'...
      '<param name=ts value=' num2str(ts) ' >\n'...
      '<param name=elements value=' num2str(elements) ' >\n'...
      '<param name=bytes value= ' num2str(bytes) ' >\n'...
      '<param name=bits value= ' num2str(bits) ' >\n'...
      '<param name=unitsx value=''pix''>\n'...
      '<param name=scalex value=''1''>\n'...
      '<param name=unitsy value=''pix''>\n'...
      '<param name=scaley value=''1''>\n'...
      '<param name=unitsz value=''pix''>\n'...
      '<param name=scalez value=''1''>\n'...
      '<param name=unitse value=''color''>\n'...
      '<param name=scalee value=''1''>\n'...
      '<param name=unitst value=''time''>\n'...
      '<param name=scalet value=''1''>\n'...
      '<param name=unitsv1 value=''int.''>\n'...
      '<param name=unitsv2 value=''int.''>\n'...
      '<param name=unitsv3 value=''int.''>\n'...
      '<param name=unitsv4 value=''int.''>\n'...
      'alt="Your browser understands the &lt;APPLET&gt; tag but is not\n' ...
      'running the applet."\n' ...
      'Your browser is completely ignoring the &lt;APPLET&gt; tag!\n' ...
      '</applet><hr></body></html>\n'];
      
fid = fopen([fn '.html'],'w');
if fid <0
   error(['Could not create file' fn '.html']);
end
fprintf(fid,content);
fclose(fid);
%work around for appletviewer problem to start with path not pwd
curdir = cd;
cd(base);
[s1,s2]=system(['appletviewer ' fn1 '.html &']);
if s1 <0
   error('Failed to start java appletviewer.');
end
cd(curdir);

%---------------------------------------------------------------------
%Installation instructions:
% put the View5D.jar in a directory of your liking and the local tmpdir
% for the time begin I put it in private
% edit matlab classpath.txt
% http://www.mathworks.com/access/helpdesk/help/techdoc/matlab_external/matlab_external.shtml
% >> which classpath.txt 
% cp to ~matlab
% append location of View5D.jar
% $HOME/DIPlib/dip/diptree/dipimage/toolbox/private/View5D.jar
